//+------------------------------------------------------------------+
//|                                               KeltnerChannel.mq4 |
//+------------------------------------------------------------------+
#property copyright ""
#property link      ""

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  White
#property indicator_color2  White
#property indicator_color3  White

//
//
//
//
//
//

extern string TimeFrame       = "current time frame";
extern int    MA_PERIOD       = 34;
extern int    MA_MODE         = 3;
extern int    PRICE_MODE      = 4;
extern int    ATR_PERIOD      = 20;
extern double K               = 1.0;
extern bool   ATR_MODE        = false;

extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = false;
extern bool   alertsOnHighLow = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;

//
//
//
//
//

double upper[];
double middle[];
double lower[];
double trend[];

//
//
//
//
//

string IndicatorFileName;
int    timeFrame;
bool   calculateValue;
bool   returningBars;

//+------------------------------------------------------------------+
//|                                                                   
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,upper);
   SetIndexBuffer(1,middle);
   SetIndexBuffer(2,lower);
   SetIndexBuffer(3,trend);
   
    //
    //
    //
    //
    //
   
      
      IndicatorFileName = WindowExpertName();
      calculateValue    = (TimeFrame=="calculateKcn");
      returningBars     = (TimeFrame=="returnBars");
      timeFrame         = stringToTimeFrame(TimeFrame);
      
    
   //
   //
   //
   //
   //
   
   IndicatorShortName(timeFrameToString(timeFrame)+  " Keltner Channels");
return(0);
}

//
//
//
//
//

int deinit() {   return(0); }

//+------------------------------------------------------------------+
//|                                                                   
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;
   
   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit=MathMin(Bars-counted_bars,Bars-1);
         if (returningBars)  { upper[0] = limit; return(0); }

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame==Period())
   {
      for(i=limit; i>=0; i--)
      {
         middle[i] = iMA(NULL,0,MA_PERIOD,0,MA_MODE,PRICE_MODE,i);

         if (ATR_MODE) double avg  = iATR(NULL,0,ATR_PERIOD,i);
         else 
         {
               double sum=0;
               for (int x=0; x<ATR_PERIOD; x++) sum += High[i+x]-Low[i+x];
                                                avg = sum/ATR_PERIOD;
         }
         
         upper[i]   = middle[i] + K * avg;
         lower[i]   = middle[i] - K * avg;
      
         trend[i] = 0;                     
            if (alertsOnHighLow)       
            {
               if (High[i] > upper[i])  trend[i] =  1;
               if (Low[i]  < lower[i])  trend[i] = -1;
            }
            else
            {
               if (Close[i] > upper[i]) trend[i] =  1;
               if (Close[i] < lower[i]) trend[i] = -1;
            }
      }
      if (!calculateValue) manageAlerts();
      return(0);            
   }
   
   //
   //
   //
   //
   //
   
   if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         upper[i]  = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateValue",MA_PERIOD,MA_MODE,PRICE_MODE,ATR_PERIOD,K,ATR_MODE,0,y);
         middle[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateValue",MA_PERIOD,MA_MODE,PRICE_MODE,ATR_PERIOD,K,ATR_MODE,1,y);
         lower[i]  = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateValue",MA_PERIOD,MA_MODE,PRICE_MODE,ATR_PERIOD,K,ATR_MODE,2,y);
         trend[i]  = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateValue",MA_PERIOD,MA_MODE,PRICE_MODE,ATR_PERIOD,K,ATR_MODE,3,y);
         
         
   }
   
   //
   //
   //
   //
   //
   
   manageAlerts();
   return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}   
 

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"lower");
         if (trend[whichBar] ==-1) doAlert(whichBar,"upper");
      }         
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," "+timeFrameToString(timeFrame)+" Keltner Channel price penetrated ",doWhat," band");
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Keltner Channel "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

